/*
    Copyright (c) 2023 Serenity Cyber Security, LLC <license@futurecrew.ru>
                Author: Gleb Popov <arrowd@FreeBSD.org>

    Redistribution and use in source and binary forms, with or without modification,
    are permitted provided that the following conditions are met:
    1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors may
    be used to endorse or promote products derived from this software without specific
    prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
    IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
    INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
    LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
    OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#pragma once

#include <memory>
#include <variant>

#include <QObject>
#include <QQueue>

#include <PolkitQt1/Authority>

#include "DBusSavedContext.h"

class QDBusContext;
class Block;
class Drive;

class AuthQueue : public QObject
{
    Q_OBJECT

public:
    using Continuation = std::function<void(PolkitQt1::Authority::Result, std::shared_ptr<DBusSavedContext>)>;

    AuthQueue();

    void Enqueue(const QString& actionId, QObject* subject, Continuation continuation);

private slots:
    void onCheckAuthorizationFinished (PolkitQt1::Authority::Result result);

private:
    using BlockOrDrive = std::variant<Block*, Drive*>;
    struct Item
    {
        QString actionId;
        BlockOrDrive bod;
        std::shared_ptr<DBusSavedContext> context;
        Continuation continuation;
    };

    void dispatchItem();

    QQueue<Item> m_items;
    PolkitQt1::Authority* m_authority;
};
